% PART OF THE CODE RELATED TO POLAR CODES IS ADAPTED FROM 
% https://ecse.monash.edu/staff/eviterbo/polarcodes.html,
% "Polar Codes for Matlab" (Vangala et al)

% ESTIMATES OF THE ENTROPY TERMS BY USING THE MONTE CARLO METHOD

clear all

% Simulation parameters:
nR = 400; % Number of realizations
n = 2^7;   % Blocklength

% CHANNEL : BS-BC (2 leg. rx and 2 eves)
Nlegch = 2;
Nevech = 2;
a2 = 0.01;
a1 = 0.04;
b2 = 0.2;
b1 = 0.35; 

% ----------------------------------------------
% Add path:
addpath('functions')

% Optimum input distribution - Individual Channels - Initial parameters for PCC:
% Criteria: max(logR1 + logR2)
% 'Channels':
%  -1-      -2-       -3-      -4-
% V-Y2  ||  V-Y1  ||   V-Z2  ||  U-Z1
%   -5-            -6-          -7-        -8-
% (V,Y2)-X  ||  (V,Z2)-X  || (V,Z1)-X  ||  V-X
p_ch = zeros(1,Nlegch+Nevech);
channelnames = cell(1,Nlegch+Nevech);
for ind = 1:Nlegch
    aux = ['a',num2str(Nlegch-ind+1)];
    auxstr = num2str(eval(aux));
    channelnames{ind} = [aux,'_',auxstr(3:min(end,7))];
    p_ch(ind) = eval(aux);
end
for ind = 1:+Nevech
    aux = ['b',num2str(Nevech-ind+1)];
    auxstr = num2str(eval(aux));
    channelnames{Nlegch+ind} = [aux,'_',auxstr(3:min(end,6))];
    p_ch(Nlegch+ind) = eval(aux);
end
% optimize distribution and initialize individual channels and parameters:
% Capacity : (R1t,R2t)
[R1t,R2t,Channels,ChannelsCell] = init_channels(p_ch,1);

NameCh = [];
for ind = 1:length(channelnames)-1
    NameCh = [NameCh,channelnames{ind},'_']; %#ok<AGROW>
end
NameCh = [NameCh,channelnames{4}];
ChannelOutput = Channels;

% Filename:
filename = ['PCC_M1_logn_',num2str(log2(n))];

% Parametres per SC encoding i decoding:
% initPolC: "Polar Codes for Matlab" (Vangala et al)
PCparams = cell(1,8);
PCparams{1} = initPolC(n,n,'BSC',ChannelOutput{1}.Py_x(1,2));
PCparams{2} = initPolC(n,n,'BSC',ChannelOutput{2}.Py_x(1,2));
PCparams{3} = initPolC(n,n,'BSC',ChannelOutput{3}.Py_x(1,2));
PCparams{4} = initPolC(n,n,'BSC',ChannelOutput{4}.Py_x(1,2));
PCparams{5} = initPolC(n,n,'BSC',ChannelOutput{5}.Py_x);
PCparams{6} = initPolC(n,n,'BSC',ChannelOutput{6}.Py_x);
PCparams{7} = initPolC(n,n,'BSC',ChannelOutput{7}.Py_x);
PCparams{8} = initPolC(n,n,'BSC',ChannelOutput{8}.Py_x(1,2));

% Compute the probabilities TERM_ul_Out = Pr (U_l (j) = 0 | U_l^{1:j-1},Out^n ),
% where l = {1,2} and Out is the corresponding output
TERM_u1_y2 = zeros(1,n); % Required to compute { H(U_1(j)|U_1^{1:j-1},Y_2^n) }_{j=1}^n
TERM_u1_y1 = zeros(1,n); % Required to compute { H(U_1(j)|U_1^{1:j-1},Y_1^n) }_{j=1}^n
TERM_u1_z2 = zeros(1,n); % Required to compute { H(U_1(j)|U_1^{1:j-1},Z_2^n) }_{j=1}^n
TERM_u1_z1 = zeros(1,n); % Required to compute { H(U_1(j)|U_1^{1:j-1},Z_1^n) }_{j=1}^n
TERM_u2_u1 = zeros(1,n); % Required to compute { H(U_2(j)|U_2^{1:j-1},V^n) }_{j=1}^n
TERM_u2_y2 = zeros(1,n); % Required to compute { H(U_2(j)|U_2^{1:j-1},V^n,Y_2^n) }_{j=1}^n
TERM_u2_z2 = zeros(1,n); % Required to compute { H(U_2(j)|U_2^{1:j-1},V^n,Z_2^n) }_{j=1}^n
TERM_u2_z1 = zeros(1,n); % Required to compute { H(U_2(j)|U_2^{1:j-1},V^n,Z_1^n) }_{j=1}^n

% ------------ Monte Carlo Method ------------ %
for ir = 1:nR,
    
    tic;
    
    % For layer 1
    u1_l1 = zeros(1,n); % v = zeros;
    % OutputOfChannel: "Polar Codes for Matlab" (Vangala et al)
    % ChannelOutput{out}.Py_x(2,1) = p_{out|V}(0|1) = p_{out|V}(1|0)
    y2_l1 = OutputOfChannel(u1_l1,'BSC',ChannelOutput{1}.Py_x(2,1));   
    y1_l1 = OutputOfChannel(u1_l1,'BSC',ChannelOutput{2}.Py_x(2,1));
    z2_l1 = OutputOfChannel(u1_l1,'BSC',ChannelOutput{3}.Py_x(2,1));
    z1_l1 = OutputOfChannel(u1_l1,'BSC',ChannelOutput{4}.Py_x(2,1));
    
    % For layer 2
    x_l2 = zeros(1,n);
    % ChannelOutput{8}.Py_x(2,1) = p_{X|V}(0|1) = p_{X|V}(1|0)
    u1_l2 = OutputOfChannel(x_l2,'BSC',ChannelOutput{8}.Py_x(2,1));
    y2_l2 = OutputOfChannel(x_l2,'BSC',a2);
    y1_l2 = OutputOfChannel(x_l2,'BSC',a1);
    z2_l2 = OutputOfChannel(x_l2,'BSC',b2);
    z1_l2 = OutputOfChannel(x_l2,'BSC',b1);
    
    term_u1_y2 = zeros(1,n);
    term_u1_y1 = zeros(1,n);
    term_u1_z2 = zeros(1,n);
    term_u1_z1 = zeros(1,n);
    term_u2_u1 = zeros(1,n);
    term_u2_y2 = zeros(1,n);
    term_u2_z2 = zeros(1,n);
    term_u2_z1 = zeros(1,n);
    
    % For each j \in [n],
    for ind2 = 1:n,
    %parfor (ind2 = 1:n,16)
        
        % For the First layer:
        % Compute probabilities from Log-Likelihood Ratios
        % poldecode: "Polar Codes for Matlab" (Vangala et al) 
        myfrozenlookup = u1_l1;
        myfrozenlookup(ind2) = -1;
        [~,llrI] = poldecode(n,y2_l1,'BSC',ChannelOutput{1}.Py_x(1,2),myfrozenlookup,PCparams{1});
        term_u1_y2(ind2) = llrI-logdomain_sum(0,llrI);
        
        myfrozenlookup = u1_l1;
        myfrozenlookup(ind2) = -1;
        [~,llrI] = poldecode(n,y1_l1,'BSC',ChannelOutput{2}.Py_x(1,2),myfrozenlookup,PCparams{2});
        term_u1_y1(ind2) = llrI-logdomain_sum(0,llrI);
        
        myfrozenlookup = u1_l1;
        myfrozenlookup(ind2) = -1;
        [~,llrI] = poldecode(n,z2_l1,'BSC',ChannelOutput{3}.Py_x(1,2),myfrozenlookup,PCparams{3});
        term_u1_z2(ind2) = llrI-logdomain_sum(0,llrI);
        
        myfrozenlookup = u1_l1;
        myfrozenlookup(ind2) = -1;
        [~,llrI] = poldecode(n,z1_l1,'BSC',ChannelOutput{4}.Py_x(1,2),myfrozenlookup,PCparams{4});
        term_u1_z1(ind2) = llrI-logdomain_sum(0,llrI);

        
        % For the Second layer:
        % Compute probabilities from Log-Likelihood Ratios
        % poldecode2: adapted from poldecode - "Polar Codes for Matlab" (Vangala et al)
        myfrozenlookup = x_l2;
        myfrozenlookup(ind2) = -1;
        [~,llrI] = poldecode(n,u1_l2,'BSC',ChannelOutput{8}.Py_x(1,2),myfrozenlookup,PCparams{8});
        term_u2_u1(ind2) = llrI-logdomain_sum(0,llrI);     
        
        myfrozenlookup2 = x_l2;
        myfrozenlookup2(ind2) = -1;
        [~,llrI] = poldecode2(n,u1_l2,y2_l2,'BSC',ChannelOutput{5}.Py_x,myfrozenlookup2,PCparams{5});
        term_u2_y2(ind2) = llrI-logdomain_sum(0,llrI);   
        
        myfrozenlookup2 = x_l2;
        myfrozenlookup2(ind2) = -1;
        [~,llrI] = poldecode2(n,u1_l2,z2_l2,'BSC',ChannelOutput{6}.Py_x,myfrozenlookup2,PCparams{6});
        term_u2_z2(ind2) = llrI-logdomain_sum(0,llrI);   
        
        myfrozenlookup2 = x_l2;
        myfrozenlookup2(ind2) = -1;
        [~,llrI] = poldecode2(n,u1_l2,z1_l2,'BSC',ChannelOutput{7}.Py_x,myfrozenlookup2,PCparams{7});
        term_u2_z1(ind2) = llrI-logdomain_sum(0,llrI);   

    end
    
    % Binary entropy terms:
    TERM_u1_y2 = TERM_u1_y2 + bh(exp(term_u1_y2));
    TERM_u1_y1 = TERM_u1_y1 + bh(exp(term_u1_y1));
    TERM_u1_z2 = TERM_u1_z2 + bh(exp(term_u1_z2));
    TERM_u1_z1 = TERM_u1_z1 + bh(exp(term_u1_z1));
    TERM_u2_u1 = TERM_u2_u1 + bh(exp(term_u2_u1));
    TERM_u2_y2 = TERM_u2_y2 + bh(exp(term_u2_y2));
    TERM_u2_z2 = TERM_u2_z2 + bh(exp(term_u2_z2));
    TERM_u2_z1 = TERM_u2_z1 + bh(exp(term_u2_z1));
    
    t = toc;
    fprintf('Done for realization %d (of %d) - %.2f seconds .\n',ir,nR,t);
    
end
% Empirical mean of Entropy Terms:
TERM_u1_y2_EST_H = TERM_u1_y2 / nR;
TERM_u1_y1_EST_H = TERM_u1_y1 / nR;
TERM_u1_z2_EST_H = TERM_u1_z2 / nR;
TERM_u1_z1_EST_H = TERM_u1_z1 / nR;
TERM_u2_u1_EST_H = TERM_u2_u1 / nR;
TERM_u2_y2_EST_H = TERM_u2_y2 / nR;
TERM_u2_z2_EST_H = TERM_u2_z2 / nR;
TERM_u2_z1_EST_H = TERM_u2_z1 / nR;

% Save results (in log domain)
% (WE USE Zs but we are saving entropies!)
ChannelOutput{1}.Polarization.Z = log(TERM_u1_y2_EST_H);
ChannelOutput{2}.Polarization.Z = log(TERM_u1_y1_EST_H);
ChannelOutput{3}.Polarization.Z = log(TERM_u1_z2_EST_H);
ChannelOutput{4}.Polarization.Z = log(TERM_u1_z1_EST_H);
ChannelOutput{5}.Polarization.Z = log(TERM_u2_y2_EST_H);
ChannelOutput{6}.Polarization.Z = log(TERM_u2_z2_EST_H);
ChannelOutput{7}.Polarization.Z = log(TERM_u2_z1_EST_H);
ChannelOutput{8}.Polarization.Z = log(TERM_u2_u1_EST_H);


save(filename,'ChannelOutput')

