% MODEL 2 - DBC-LD-NLS
% Definition of the partition of the universal set [n] and 
% performance evaluation

clear all

% Simulation parameters:
nV = 2.^([7,8,9,10,11,12,13,14,15,16]); %blocklengths
betaS_1 = 0.30;  % \beta^{(1,\text{s})}
betaS_2 = 0.36;  % \beta^{(2,\text{s})}
betaR_1 = 0.18;  % \beta^{(1,\text{r})}
betaR_2 = 0.18;  % \beta^{(2,\text{r})}
betaH_2 = 0.36; % \beta^{(2,\text{H})}
betaL_2 = 0.36; % \beta^{(2,\text{L})}
rho_R = 80;   % \rho_{\text{R}}
nrelTV = 200;    % Number of realizations to compute the impact of the deterministic SC encoding on the total variation distance


% Load files -> Contain the estimates of the corresponding entropy terms
% CH{ind}{i}, where i \in [1,8].
    % i determines the "channel":
    %  -1-      -2-       -3-      -4-
    % U|Y2  ||  U|Y1  ||   U|Z2  ||  U|Z1
    %   -5-            -6-          -7-        -8-
    % (U,Y2)|X  ||  (U,Z2)|X  || (U,Z1)|X  ||  U|X
% CH{ind}{i}.Polarization.Z -> log of ENTROPY TERMS, not Bhattacharyya par.
CH = cell(1,length(nV));
load('Simulated_PCC/PCC_M2_logn_7.mat')
CH{1} = ChannelOutput;
load('Simulated_PCC/PCC_M2_logn_8.mat')
CH{2} = ChannelOutput;
load('Simulated_PCC/PCC_M2_logn_9.mat')
CH{3} = ChannelOutput;
load('Simulated_PCC/PCC_M2_logn_10.mat')
CH{4} = ChannelOutput;
load('Simulated_PCC/PCC_M2_logn_11.mat')
CH{5} = ChannelOutput;
load('Simulated_PCC/PCC_M2_logn_12.mat')
CH{6} = ChannelOutput;
load('Simulated_PCC/PCC_M2_logn_13.mat')
CH{7} = ChannelOutput;
load('Simulated_PCC/PCC_M2_logn_14.mat')
CH{8} = ChannelOutput;
load('Simulated_PCC/PCC_M2_logn_15.mat')
CH{9} = ChannelOutput;
load('Simulated_PCC/PCC_M2_logn_16.mat')
CH{10} = ChannelOutput;

% Add path:
addpath('functions')

% CHANNEL - Broadcast BSC (2 leg. receivers and 2 eavesdroppers)
Nlegch = 2;
Nevech = 2;
a2 = 0.01;
a1 = 0.04;
b2 = 0.2;
b1 = 0.35; 

% Target rates and channel initialization:
p_ch = [a2,a1,b2,b1];
[R1t,R2t,~,~] = init_channels(p_ch,1);
R1targ = (rho_R/100) * R1t;
R2targ = (rho_R/100) * R2t;

% File name (where results are stored)
filename = ['Simulated_Results/M2_PercentRate_',num2str(rho_R),...
    '_betaS_1(percent)_',num2str(betaS_1*100),'_betaR_1(percent)_',num2str(betaR_1*100),...
    '_betaS_2(percent)_',num2str(betaS_2*100),'_betaR_2(percent)_',num2str(betaR_2*100),...
    '_betaH_2(percent)_',num2str(betaH_2*100),'_betaL_2(percent)_',num2str(betaL_2*100),...
    '_from_',num2str(log2(nV(1))),'_to_',num2str(log2(nV(end)))];

% Total variation distance (upper-bound):
TVdist_L = zeros(1,length(nV));
TVdist_H = zeros(1,length(nV));

% Information leakages (upper bound):
I_I1_Z2 = zeros(1,length(nV));
I_I1_Z1 = zeros(1,length(nV));
I_F1_Z2 = zeros(1,length(nV));
I_F1_Z1 = zeros(1,length(nV));
I_I1F1_Z2 = zeros(1,length(nV));
I_I1F1_Z1 = zeros(1,length(nV));
I_I2_Z2 = zeros(1,length(nV));
I_I2_Z1 = zeros(1,length(nV));
I_F2_Z2 = zeros(1,length(nV));
I_F2_Z1 = zeros(1,length(nV));
I_I2F2_Z2 = zeros(1,length(nV));
I_I2F2_Z1 = zeros(1,length(nV));
I_I1I2_Z2 = zeros(1,length(nV));
I_I1I2_Z1 = zeros(1,length(nV));
I_I1I2F1F2_Z2 = zeros(1,length(nV));
I_I1I2F1F2_Z1 = zeros(1,length(nV));

% Average bit error probabilities (upper bound)
Pe_I1_Y1 = zeros(1,length(nV));
Pe_I1I2_Y2 = zeros(1,length(nV));

% Rate of the additional secret sequence \Phi
SSR = zeros(1,length(nV));
SSR1 = zeros(1,length(nV));
SSR2 = zeros(1,length(nV));

for ind = 1:length(nV),
    
    tic; 
    n = nV(ind); 
    
    % delta_ns (function of betas)
    deltaS_1 = (-n^(betaS_1))*log(2);
    deltaS_2 = (-n^(betaS_2))*log(2);
    deltaR_1 = (-n^(betaR_1))*log(2);
    deltaR_2 = (-n^(betaR_2))*log(2);
    deltaH_2 = (-n^(betaH_2))*log(2);
    deltaL_2 = (-n^(betaL_2))*log(2);
    
    % Partition of the universal set [n]
    % ISSi (i=1,2) -> indices corresponding to \Phi_i
    % T2 is a structure: T2.L = \mathcal{L}_{X|V}^{(n)} and T2.NP = setdifff(1:1:n,union(T2.L,H2))
    % where H2 = \mathcal{H}_{X|V}^{(n)} 
    [I1,C1,F1,I2,C2,F2,T2,ISS1,ISS2] = define_sets_new_log(CH{ind},ceil(R1targ*n),...
        ceil(R2targ*n),deltaH_2,deltaL_2,deltaS_1,deltaS_2,deltaR_1,deltaR_2);
    H2 = union(union(F2,I2),C2);

    % Distortion introduced by the encoding (upper-bound TV distance):
    % Only on layer 2!
    % CH{ind}{8}.Py_x(1,2) = \alpha_{X|V}
    % TVdist_L -> impact of the deterministic SC encoding on the TVdist
    % TVdist_H -> impact on the TVdist of storing uniformly distributed random sequences into \tilde{U}[H_{X|V}^{n}]
    TVdist_L(ind) = totalvariationL(n,H2,T2,CH{ind}{8}.Py_x(1,2),nrelTV); 
    TVdist_H(ind) = vpa(vpa(sqrt(2*log(2)))*vpa(sqrt(vpa(length(H2)-sum(vpa(exp(vpa(CH{ind}{8}.Polarization.Z(H2)))))))));
    
    % Upper-Bound Error Probabilities:
    S1 = union(C1,I1);
    S2 = union(C2,union(I2,union(T2.L,T2.NP)));
    S1minusISS1 = setdiff(S1,ISS1);
    S2minusISS2 = setdiff(S2,ISS2);
    Pe_I1_Y1(ind) = sum(sqrt(exp(CH{ind}{2}.Polarization.Z(S1minusISS1))))/(length(S1minusISS1));
    Pe_I1I2_Y2(ind) = sum(sqrt(exp(CH{ind}{1}.Polarization.Z(S1minusISS1))))/length(S1minusISS1) ...
        + sum(sqrt(exp(CH{ind}{5}.Polarization.Z(S2minusISS2))))/length(S2minusISS2);
    
    % Information leakage:
    I_I1_Z2(ind) = length(I1) - sum(exp(CH{ind}{3}.Polarization.Z(I1)));
    I_I1_Z1(ind) = length(I1) - sum(exp(CH{ind}{4}.Polarization.Z(I1)));
    I_F1_Z2(ind) = length(F1) - sum(exp(CH{ind}{3}.Polarization.Z(F1)));
    I_F1_Z1(ind) = length(F1) - sum(exp(CH{ind}{4}.Polarization.Z(F1)));
    I_I1F1_Z2(ind) = length(I1)+length(F1) - sum(exp(CH{ind}{3}.Polarization.Z(union(I1,F1))));
    I_I1F1_Z1(ind) = length(I1)+length(F1) - sum(exp(CH{ind}{4}.Polarization.Z(union(I1,F1))));
    I_I2_Z2(ind) = length(I2) - sum(exp(CH{ind}{6}.Polarization.Z(I2)));
    I_I2_Z1(ind) = length(I2) - sum(exp(CH{ind}{7}.Polarization.Z(I2)));
    I_F2_Z2(ind) = length(F2) - sum(exp(CH{ind}{6}.Polarization.Z(F2)));
    I_F2_Z1(ind) = length(F2) - sum(exp(CH{ind}{7}.Polarization.Z(F2)));
    I_I2F2_Z2(ind) = length(I2)+length(F2) - sum(exp(CH{ind}{6}.Polarization.Z(union(I2,F2))));
    I_I2F2_Z1(ind) = length(I2)+length(F2) - sum(exp(CH{ind}{7}.Polarization.Z(union(I2,F2))));
    I_I1I2_Z2(ind) = length(I1)+length(I2) - sum(exp(CH{ind}{3}.Polarization.Z(I1)))-sum(exp(CH{ind}{6}.Polarization.Z(I2)));
    I_I1I2_Z1(ind) = length(I1)+length(I2) - sum(exp(CH{ind}{4}.Polarization.Z(I1)))-sum(exp(CH{ind}{7}.Polarization.Z(I2)));
    I_I1I2F1F2_Z2(ind) = length(I1)+length(I2)+length(F1)+length(F2)...
        - sum(exp(CH{ind}{3}.Polarization.Z(I1)))-sum(exp(CH{ind}{6}.Polarization.Z(I2)))...
        - sum(exp(CH{ind}{3}.Polarization.Z(F1)))-sum(exp(CH{ind}{6}.Polarization.Z(F2)));
    I_I1I2F1F2_Z1(ind) = length(I1)+length(I2)+length(F1)+length(F2)...
        - sum(exp(CH{ind}{4}.Polarization.Z(I1)))-sum(exp(CH{ind}{7}.Polarization.Z(I2)))...
        - sum(exp(CH{ind}{4}.Polarization.Z(F1)))-sum(exp(CH{ind}{7}.Polarization.Z(F2)));    

    % Additional secret sequence rates:
    SSR1(ind) = length(ISS1)/n;
    SSR2(ind) = length(ISS2)/n;
    SSR(ind) = (length(ISS1) + length(ISS2))/n;
    

    t = toc;
    fprintf('Done for n=2^N, where N equal to %d - %.2f seconds .\n',log2(n),t);
    
end
save(filename,'Pe_I1_Y1','Pe_I1I2_Y2','SSR','SSR1','SSR2',...
    'I_I1F1_Z2','I_I1F1_Z1','I_I2F2_Z2','I_I2F2_Z1','I_I1I2F1F2_Z2','I_I1I2F1F2_Z1',...
    'TVdist_H','TVdist_L');







