% PART OF THE CODE RELATED TO POLAR CODES IS ADAPTED FROM 
% https://ecse.monash.edu/staff/eviterbo/polarcodes.html,
% "Polar Codes for Matlab" (Vangala et al)
%
% Function that computes Eq.(52) of the paper
%

function V = totalvariationL(n,H2,T2,P_XV,nR)

digits(128);
PCparams = initPolC(n,length(union(T2.L,T2.NP)),'BSC',P_XV);

ValAuxSUM = vpa(zeros(1,length(T2.L)));

for ir = 1:nR,
    
    tic;
    
    % Encoding:
    % ---------
    % pencode: "Polar Codes for Matlab" (Vangala et al)
    u1 = zeros(1,n);
    v1 = pencode(log2(n),[],u1);

    u2 = zeros(1,n);
    u2(H2) = rand(1,length(H2))<=0.5;
    
    % Random and Deterministic SC encoding:
    myfrozenlookup2 = zeros(1,n);
    myfrozenlookup2(H2) = u2(H2);
    myfrozenlookup2(union(T2.NP,T2.L)) = -1;
    % pencode_SC -> draws \check{U}_2^n according to Eq.(A2):
    % pencode_SC: adapted from pencode "Polar Codes for Matlab" (Vangala et al)
    u2 = pencode_SC(n,v1','BSC',P_XV,myfrozenlookup2,PCparams,T2);
    
    % Monte Carlo Method
    for ind = 1:length(T2.L),
    %parfor (ind = 1:length(T2.L),16)
        index_j = T2.L(ind);
        myfrozenlookup = u2;
        myfrozenlookup(index_j) = -1;
        [~,llrI] = poldecode(n,v1','BSC',P_XV,myfrozenlookup,PCparams);
        Probs_0 = vpa(exp(vpa(llrI-logdomain_sum(0,llrI))));
        Probs_1 = vpa(1-Probs_0);
        M = vpa([Probs_0; Probs_1]);
        ValAuxSUM(ind) = vpa(ValAuxSUM(ind) + vpa((1 - max(M))));                
    end
    t = toc;
    fprintf('TOTAL VARIATION -> Done for realization %d (of %d) - %.2f seconds .\n',ir,nR,t);

end
V = double(vpa(sum(vpa(ValAuxSUM/nR))));
